//-----------------------------------------------------------------------------
// Torque Game Engine 
// Copyright (C) GarageGames.com, Inc.
//-----------------------------------------------------------------------------


//-----------------------------------------------------------------------------
//  Server and mission initialization
//-----------------------------------------------------------------------------

function onServerCreated()
{
   // This function is called when a server is constructed.

   // Master server information for multiplayer games
   $Server::GameType = "Torque TTB";
   $Server::MissionType = "None";

   // Load up all datablocks, objects etc.
   exec("./camera.cs");
   exec("./editor.cs");
   exec("./player.cs");
   exec("./items.cs");
   exec("./audioProfiles.cs");
   exec("./particle.cs");

   exec("common/server/lightingSystem.cs");
}

function onServerDestroyed()
{
   // This function is called as part of a server shutdown.
}


//-----------------------------------------------------------------------------

function onMissionLoaded()
{
   // Called by loadMission() once the mission is finished loading.
}

function onMissionEnded()
{
   // Called by endMission(), right before the mission is destroyed
}


//-----------------------------------------------------------------------------
// Dealing with client connections
// These methods are extensions to the GameConnection class. Extending
// GameConnection make is easier to deal with some of this functionality,
// but these could also be implemented as stand-alone functions.
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------

function GameConnection::onClientEnterGame(%this)
{
   // Every client get's a camera object.
   %this.camera = new Camera() {
      dataBlock = Observer;
   };
   MissionCleanup.add( %this.camera );
   %this.camera.scopeToClient(%this);

   // Create a player object.
   %spawnPoint = pickSpawnPoint();
   %this.createPlayer(%spawnPoint);
}

function GameConnection::onClientLeaveGame(%this)
{
   if (isObject(%this.camera))
      %this.camera.delete();
   if (isObject(%this.player))
      %this.player.delete();
}


//-----------------------------------------------------------------------------

$ActorNum = 1;

function GameConnection::createPlayer(%this, %spawnPoint)
{
   if (%this.player > 0)  {
      // The client should not have a player currently
      // assigned.  Assigning a new one could result in 
      // a player ghost.
      error( "Attempting to create an angus ghost!" );
   }

   // Create the player object
   %player = new Player() {
      dataBlock = PlayerBody;
      client = %this;
   };
   MissionCleanup.add(%player);

   // Player setup...
   %player.setTransform("95.6 -29.1 111.608 0 0 1 3.14159");
   %player.playthread(1,"laydown");
   //%player.setShapeName(%this.name);
   spawngoat();
   
   // Update the camera to start with the player
   %this.camera.setTransform("92.7946 -28.8934 112.248 -0.0434143 0.0438493 0.998094 1.58267");

   // Give the client control of the player
   %this.player = %player;
   %this.setControlObject(%this.camera);
   $Actor[0] = %player;
   $Camera = %this.camera;
   $Client = %this;
}

function spawngoat()
{
   %spawnPoint = pickSpawnPoint();
   // Create the aiplayer object
   %goat = new AIPlayer() {
      dataBlock = goat;
   };
   MissionCleanup.add(%goat);

   // Player setup...
   %goat.setTransform("95.2 -29 111.608 1 0 0 0");
   %goat.setScale("0.75 0.75 0.75");
   %goat.playthread(1,"laydown");
   $Actor[1] = %goat;
}

function servercmdfixmyshit(%this)
{
	$Actor[0].setTransform("95.6 -29.1 111.608 0 0 1 3.14159");
	$Actor[1].setTransform("95.2 -29 111.608 1 0 0 0");
	$Camera.setTransform("92.7946 -28.8934 112.248 -0.0434143 0.0438493 0.998094 1.58267");
}

$AnimalNum = -1;
$Animal[$AnimalNum++] = Goat;
$Animal[$AnimalNum++] = Sheep;
$Animal[$AnimalNum++] = Pig;
$Animal[$AnimalNum++] = Osama;
$Animal[$AnimalNum++] = Male;

$CurAnimal = 0;

function servercmdNextAnimal(%this)
{
	$CurAnimal++;
	if( $CurAnimal > $AnimalNum )
	{
		$CurAnimal = 0;
	}
	
	while( $ActorNum > 0 )
	{
		$Actor[$ActorNum].delete();
		$ActorNum--;
	}
	
	$Actor[0].setTransform(%spawnPoint);
	$Actor[0].stopthread(1);
	
	%animal = new AIPlayer() {
		dataBlock = $Animal[$CurAnimal];
	};
	
	MissionCleanup.add(%animal);
	%animal.setTransform("95.5835 -28.9293 111.608 1 0 0 0");
	%animal.setScale($Animal[$CurAnimal].defaultScale);
	%animal.playthread(1,"prime");
	$Actor[$ActorNum++] = %animal;
	//$Client.setFirstPerson(false);
	CamMooring.setTransform($Animal[$CurAnimal].camTrans);
	$Camera.setOrbitMode(CamMooring, $Camera.gettransform(), 1, 3, 3);
	
	commandtoclient(%this,'SetAnimalName',$CurAnimal);
}

function servercmdPrevAnimal(%this)
{
	$CurAnimal--;
	if( $CurAnimal < 0 )
	{
		$CurAnimal = $AnimalNum;
	}
	
	while($ActorNum > 0)
	{
		$Actor[$ActorNum].delete();
		$ActorNum--;
	}
	
	$Actor[0].setTransform(%spawnPoint);
	$Actor[0].stopthread(1);
	
	%animal = new AIPlayer() {
		dataBlock = $Animal[$CurAnimal];
	};
	
	MissionCleanup.add(%animal);
	%animal.setTransform("95.5835 -28.9293 111.608 1 0 0 0");
	%animal.setScale($Animal[$CurAnimal].defaultScale);
	%animal.playthread(1,"prime");
	$Actor[$ActorNum++] = %animal;
	//$Client.setFirstPerson(false);
	CamMooring.setTransform($Animal[$CurAnimal].camTrans);
	$Camera.setOrbitMode(CamMooring, $Camera.gettransform(), 1, 3, 3);
	
	commandtoclient(%this,'SetAnimalName',$CurAnimal);
}

function servercmdPrimeAnimalSelect(%this)
{
	%spawnPoint = pickSpawnPoint();

	while($ActorNum > 0)
	{
		$Actor[$ActorNum].delete();
		$ActorNum--;
	}
	
	$Actor[0].setTransform(%spawnPoint);
	$Actor[0].stopthread(1);
	
	%goat = new AIPlayer() {
      dataBlock = goat;
	};
	
	MissionCleanup.add(%goat);
	%goat.setTransform("95.5835 -28.9293 111.608 1 0 0 0");
	%goat.setScale("0.75 0.75 0.75");
	%goat.playthread(1,"prime");
	$Actor[$ActorNum++] = %goat;
	//$Client.setFirstPerson(false);
	$Camera.setOrbitMode(CamMooring, "92.7946 -28.8934 112.248 -0.0434143 0.0438493 0.998094 1.58267", 1, 3, 3);
	$Camera.gizMode = 0;
	
	$CurAnimal = 0;
	CamMooring.setTransform($Animal[$CurAnimal].camTrans);
	commandtoclient(%this,'SetAnimalName',$CurAnimal);
}

function servercmdstartsex(%this)
{
	//This command will fill out the client's sex interface and move everybody into their default idle positions
	$HasCum = 0;
	$CurSeq = 0;
	$Actor[0].setTransform($Animal[$CurAnimal].sexPlayerTrans[0]);
	$Actor[1].setTransform($Animal[$CurAnimal].sexTrans[0]);
	$Actor[0].playthread(1,$Animal[$CurAnimal].sexPlayerAnim[0]);
	$Actor[1].playthread(1,$Animal[$CurAnimal].sexAnim[0]);
	if($Animal[$CurAnimal].sexSound[0] !$= "")
		$Actor[1].playAudio(1, $Animal[$CurAnimal].sexSound[0]);
	if($Animal[$CurAnimal].sexMoan[0] !$= "")
		doMoan();
	%var = $Animal[$CurAnimal].sexNum;
	for(%i = 0; %i < $Animal[$CurAnimal].sexNum; %i ++)
	{
		%var = %var SPC $Animal[$CurAnimal].sexButton[%i];
	}
	//echo(%var);
	$Camera.setOrbitMode(CamMooring,$Camera.gettransform(), 1, 3, 3);
	commandtoclient(%this,'sexmenu',%var);
	commandtoclient(%this,'EjacButton',0);
	
	//group sex
	if($Animal[$CurAnimal].sexGang > 0)
	{
		for(%i = 0; %i < $Animal[$CurAnimal].sexGang; %i ++)
		{
			spawnExtra(%i);
			//echo(%i);
		}
	}
}

function servercmddosex(%this,%num)
{
	$Actor[0].hasParticle = 0;
	$Actor[1].hasParticle = 0;
	$Actor[0].unmountImage(0);
	$Actor[1].unmountImage(0);
	$HasCum = 0;
	$Actor[0].playthread(1,"prime");
	$Actor[1].playthread(1,"prime");
	$Actor[0].setTransform($Animal[$CurAnimal].sexPlayerTrans[%num]);
	$Actor[1].setTransform($Animal[$CurAnimal].sexTrans[%num]);
	$Actor[0].schedule(64,playthread,1,$Animal[$CurAnimal].sexPlayerAnim[%num]);
	$Actor[1].schedule(64,playthread,1,$Animal[$CurAnimal].sexAnim[%num]);
	if($Animal[$CurAnimal].sexSound[%num] !$= "")
		$Actor[1].playAudio(1, $Animal[$CurAnimal].sexSound[%num]);
	if($Animal[$CurAnimal].sexMoan[%num] !$= "")
		doMoan();
	if($Animal[$CurAnimal].sexPee[%num] != 0)
	{
		$Actor[1].hasParticle = 1;
		$Actor[1].mountImage(PissImage,0);
		doParticle($Animal[$CurAnimal].sexPeeDelay[%num], PissImage, $Actor[1], 0, $Animal[$CurAnimal].sexPee[%num], $Animal[$CurAnimal].sexPeeTrans[%num]);
	}
	if($Animal[$CurAnimal].sexPoop[%num] != 0)
	{
		$Actor[1].hasParticle = 1;
		$Actor[1].mountImage(ShitImage,0);
		doParticle($Animal[$CurAnimal].sexPoopDelay[%num], ShitImage, $Actor[1], 0, $Animal[$CurAnimal].sexPoop[%num], $Animal[$CurAnimal].sexPoopTrans[%num]);
	}
	
	$CurSeq = %num;
	if(%num > 0)
		commandtoclient(%this,'EjacButton',1);
	else
		commandtoclient(%this,'EjacButton',0);
	
	//group sex
	if($Animal[$CurAnimal].sexGang > 0)
	{
		for(%i = 0; %i < $Animal[$CurAnimal].sexGang; %i ++)
		{
			%wat = %i + 2;
			//echo(%wat);
			$Actor[%wat].hasParticle = 0;
			$Actor[%wat].unmountImage(0);
			$Actor[%wat].playthread(1,"prime");
			$Actor[%wat].settransform(getField($Animal[$CurAnimal].sexExtraTrans[$CurSeq],%i));
			$Actor[%wat].schedule(64,playthread,1,getField($Animal[$CurAnimal].sexExtraAnim[$CurSeq],%i));
			if($Animal[$CurAnimal].sexPeeExtraTrans[%num] !$= "")
			{
				$Actor[%wat].hasParticle = 1;
				$Actor[%wat].mountImage(PissImage,0);
				doParticle($Animal[$CurAnimal].sexPeeDelay[%num], PissImage, $Actor[%wat], 0, getField($Animal[$CurAnimal].sexPeeExtra[%num],%i), getField($Animal[$CurAnimal].sexPeeExtraTrans[%num],%i));
			}
			if($Animal[$CurAnimal].sexPoopExtraTrans[%num] !$= "")
			{
				$Actor[%wat].hasParticle = 1;
				$Actor[%wat].mountImage(ShitImage,0);
				doParticle($Animal[$CurAnimal].sexPoopDelay[%num], ShitImage, $Actor[%wat], 0, getField($Animal[$CurAnimal].sexPoopExtra[%num],%i), getField($Animal[$CurAnimal].sexPoopExtraTrans[%num],%i));
			}
		}
	}
}

function servercmdCameraShit(%this)
{
	//This function toggles the camera between an orbit cam, and a type of eye cam.
	if($Camera.gizMode == 1)
	{
		$Camera.gizMode = 0;
		$Camera.setOrbitMode(CamMooring, $Camera.gettransform(), 1, 3, 3);
	}
	else
	{
		$Camera.gizMode = 1;
		$Camera.setOrbitMode($Actor[0], $Actor[0].gettransform(), 0, 0, 0);
	}
}

$moanNum = -1;
$moanArray[$moanNum++] = "malemoan";
$moanArray[$moanNum++] = "malemoan2";
$moanArray[$moanNum++] = "malemoan3";
$moanArray[$moanNum++] = "malemoan4";

function servercmdMakeMeCum(%this)
{
	if($HasCum != 1)
	{
		$HasCum = 1;
		$CumNum = 0;
		$Actor[0].isCumming = 0;
		if($Animal[$CurAnimal].sex[$CurSeq] $= "bukkake")
		{
			$Actor[1].mountImage(CumImage,0);
			if($Animal[$CurAnimal].sexGang > 0)
			{
				for(%i = 0; %i < $Animal[$CurAnimal].sexGang; %i ++)
				{
				%wat = %i + 2;
				$Actor[%wat].mountImage(CumImage,0);
				}
			}
			//Not the most efficient way to do these two things but idgaf, I'm close to release
		}
		if($Animal[$CurAnimal].sexGang > 0)
			%val = 1;
		else
			%val = 0;
		commandToClient(%this,'PlayCumshot',%val);
		doCum();
	}
}

function doCum()
{
	if($Actor[0].isCumming != 1)
	{
		if($CumNum < 3)
		{
			$Actor[0].isCumming = 1;
			$Actor[0].pauseThread(1);
			$Actor[1].pauseThread(1);
			$Actor[1].stopAudio(1);
			$Actor[0].playAudio(3, cum);
			$Actor[0].playAudio(2, malecum);
			if($Animal[$CurAnimal].sex[$CurSeq] $= "bukkake")
			{
				fireaway(CumImage, $Actor[1], 0, 3, $Animal[$CurAnimal].sexCumTrans[$CurSeq]);
				fireaway(CumImage, $Actor[1], 0, 3, $Animal[$CurAnimal].sexCumTrans[$CurSeq]);
				fireaway(CumImage, $Actor[1], 0, 3, $Animal[$CurAnimal].sexCumTrans[$CurSeq]);
				fireaway(CumImage, $Actor[1], 0, 3, $Animal[$CurAnimal].sexCumTrans[$CurSeq]);
				fireaway(CumImage, $Actor[1], 0, 3, $Animal[$CurAnimal].sexCumTrans[$CurSeq]);
			}
			if($Animal[$CurAnimal].sexMoan[$CurSeq] !$= "")
			{
				%sound = $Animal[$CurAnimal].sexMoan[$CurSeq];
				if(%sound $= "malemoan")
					%sound = $moanArray[getrandom(0,3)];
				$Actor[1].PlayAudio(2, %sound);
			}
			schedule(1000,0,doCum);
			$CumNum++;
			if($Animal[$CurAnimal].sexGang > 0)
			{
				for(%i = 0; %i < $Animal[$CurAnimal].sexGang; %i ++)
				{
					%wat = %i + 2;
					//echo(%wat);
					$Actor[%wat].pauseThread(1);
					if($Animal[$CurAnimal].sexMoan[$CurSeq] !$= "")
					{
						%sound = $Animal[$CurAnimal].sexMoan[$CurSeq];
						if(%sound $= "malemoan")
							%sound = $moanArray[getrandom(0,3)];
						$Actor[%wat].PlayAudio(2, %sound);
					}
					if($Animal[$CurAnimal].sex[$CurSeq] $= "bukkake")
					{
						echo("I'M CUMMING!");
						fireaway(CumImage, $Actor[%wat], 0, 3, getField($Animal[$CurAnimal].sexCumExtraTrans[$CurSeq],%i));
						fireaway(CumImage, $Actor[%wat], 0, 3, getField($Animal[$CurAnimal].sexCumExtraTrans[$CurSeq],%i));
						fireaway(CumImage, $Actor[%wat], 0, 3, getField($Animal[$CurAnimal].sexCumExtraTrans[$CurSeq],%i));
						fireaway(CumImage, $Actor[%wat], 0, 3, getField($Animal[$CurAnimal].sexCumExtraTrans[$CurSeq],%i));
						fireaway(CumImage, $Actor[%wat], 0, 3, getField($Animal[$CurAnimal].sexCumExtraTrans[$CurSeq],%i));
					}
				}
			}
		}
		else
		{
			$Actor[0].playThread(1,faint);
			$Actor[1].playThread(1,faint);
			if($Animal[$CurAnimal].sexGang > 0)
			{
				for(%i = 0; %i < $Animal[$CurAnimal].sexGang; %i ++)
				{
					%wat = %i + 2;
					//echo(%wat);
					$Actor[%wat].playThread(1,faint);
				}
			}
		}
	}
	else
	{
		if($CumNum < 3)
		{
			$Actor[0].isCumming = 0;
			$Actor[0].playThread(1,$Animal[$CurAnimal].sexPlayerAnim[$CurSeq]);
			$Actor[1].playthread(1,$Animal[$CurAnimal].sexAnim[$CurSeq]);
			if($Animal[$CurAnimal].sexGang > 0)
			{
				for(%i = 0; %i < $Animal[$CurAnimal].sexGang; %i ++)
				{
					%wat = %i + 2;
					//echo(%wat);
					$Actor[%wat].playThread(1,getField($Animal[$CurAnimal].sexExtraAnim[$CurSeq],%i));
				}
			}
			schedule(1000,0,doCum);
		}
		else
		{
			$Actor[0].playThread(1,faint);
			$Actor[1].playThread(1,faint);
			if($Animal[$CurAnimal].sexGang > 0)
			{
				for(%i = 0; %i < $Animal[$CurAnimal].sexGang; %i ++)
				{
					%wat = %i + 2;
					//echo(%wat);
					$Actor[%wat].playThread(1,faint);
				}
			}
		}
	}
}

function doMoan()
{
	//echo("I have been told to moan");
	cancel($Moan);
	if($HasCum != 1)
	{
		%rand = getrandom(0,2);
		if(%rand == 0)
		{
			//echo("I am moaning");
			//echo($Animal[$CurAnimal].sexMoan[$CurSeq]);
			%sound = $Animal[$CurAnimal].sexMoan[$CurSeq];
			if(%sound $= "malemoan")
				%sound = $moanArray[getrandom(0,3)];
			$Actor[1].PlayAudio(2, %sound);
		}
		//group sex
		if($Animal[$CurAnimal].sexGang > 0)
		{
			%rand2 = getrandom(0,2);
			if(%rand2 == 0)
			{
				%sound = $Animal[$CurAnimal].sexMoan[$CurSeq];
				if(%sound $= "malemoan")
					%sound = $moanArray[getrandom(0,3)];
				$Actor[getrandom(2, $Animal[$CurAnimal].sexGang + 1)].PlayAudio(2, %sound);
			}
		}
		$Moan = schedule(750,0,doMoan);
	}
}

function spawnExtra(%num)
{
	%animal = new AIPlayer() {
		dataBlock = $Animal[$CurAnimal];
	};
	
	MissionCleanup.add(%animal);
	%animal.setTransform(getField($Animal[$CurAnimal].sexExtraTrans[$CurSeq],%num));
	%animal.setScale($Animal[$CurAnimal].defaultScale);
	%animal.playthread(1,"prime");
	$Actor[$ActorNum++] = %animal;
}

function doParticle(%delay, %this, %obj, %slot, %vel, %trans)
{
	if(%obj.hasParticle == 1 && $HasCum != 1)
	{
		cancel(%obj.partSch);
		fireaway(%this, %obj, %slot, %vel, %trans);
		%obj.partSch = schedule(%delay, 0, doParticle, %delay, %this, %obj, %slot, %vel, %trans);
	}
}

//-----------------------------------------------------------------------------

function pickSpawnPoint() 
{
   // Pick the first object in drop point group and use it's
   // location as a spawn point.
   %group = nameToID("MissionGroup/PlayerDropPoints");
   if (%group != -1 && %group.getCount() != 0)
      return %group.getObject(0).getTransform();

   // If no object was found, return a point near the center of the world
   error("Missing spawn point object and/or mission group " @ %groupName);
   return "0 0 300 1 0 0 0";
}